@@ -344,7 +344,7 @@ binary_function(PyObject *args, enum kernel_type kern, const char *format)
 
 #define COUNT_FUNC(oper, ochar)                                         \
 static PyObject *                                                       \
-count_ ## oper (bitarrayobject *module, PyObject *args)                 \
+count_ ## oper (PyObject *module, PyObject *args)                       \
 {                                                                       \
     return binary_function(args, KERN_c ## oper, "OO:count_" #oper);    \
 }                                                                       \
@@ -451,6 +451,7 @@ ba2hex(PyObject *module, PyObject *a)
         str[i + le] = hexdigits[c >> 4];
         str[i + be] = hexdigits[0x0f & c];
     }
+    assert((size_t) aa->nbits / 4 <= strsize);
     result = Py_BuildValue("s#", str, aa->nbits / 4);
 #undef aa
     PyMem_Free((void *) str);
@@ -497,11 +498,11 @@ hex2ba(PyObject *module, PyObject *args)
             /* there is an invalid byte - or (non-terminating) NUL */
             if (x < 0 || y < 0) {
                 PyErr_SetString(PyExc_ValueError,
-                                "Non-hexadecimal digit found");
+                                "non-hexadecimal digit found");
                 return NULL;
             }
         }
-        assert(x < 16 && y < 16);
+        assert(0 <= x && x < 16 && 0 <= y && y < 16);
         aa->ob_item[i / 2] = x << 4 | y;
     }
 #undef aa
@@ -520,19 +521,17 @@ static const char base64_alphabet[] =
 static int
 digit_to_int(char c, int n)
 {
-    if (n <= 16) {              /* base 2, 4, 8, 16 */
-        int i = hex_to_int(c);
+    int i;
 
-        if (0 <= i && i < n)
-            return i;
-    }
-    if (n == 32) {              /* base 32 */
+    switch (n) {
+    case 32:                    /* base 32 */
         if ('A' <= c && c <= 'Z')
             return c - 'A';
         if ('2' <= c && c <= '7')
             return c - '2' + 26;
-    }
-    if (n == 64) {              /* base 64 */
+        break;
+
+    case 64:                    /* base 64 */
         if ('A' <= c && c <= 'Z')
             return c - 'A';
         if ('a' <= c && c <= 'z')
@@ -543,6 +542,12 @@ digit_to_int(char c, int n)
             return 62;
         if (c == '/')
             return 63;
+        break;
+
+    default:                    /* base 2, 4, 8, 16 */
+        i = hex_to_int(c);
+        if (i < n)
+            return i;
     }
     return -1;
 }
@@ -557,7 +562,8 @@ base_to_length(int n)
         if (n == (1 << m))
             return m;
     }
-    PyErr_SetString(PyExc_ValueError, "base must be 2, 4, 8, 16, 32 or 64");
+    PyErr_Format(PyExc_ValueError,
+                 "base must be 2, 4, 8, 16, 32 or 64, not %d", n);
     return -1;
 }
 
@@ -629,15 +635,17 @@ standard base 64 alphabet is used.");
 static PyObject *
 base2ba(PyObject *module, PyObject *args)
 {
-    PyObject *a;
-    Py_ssize_t i, strsize;
-    char *str;
+    PyObject *a = NULL;
+    Py_ssize_t i, strsize = 0;
+    char *str = NULL;
     int n, m, le;
 
-    if (!PyArg_ParseTuple(args, "iOs#", &n, &a, &str, &strsize))
+    if (!PyArg_ParseTuple(args, "i|Os#", &n, &a, &str, &strsize))
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
+    if (a == NULL)
+        return PyLong_FromLong(m);
     if (ensure_ba_of_length(a, m * strsize) < 0)
         return NULL;
 
@@ -649,8 +657,9 @@ base2ba(PyObject *module, PyObject *args)
         int j, k, d = digit_to_int(str[i], n);
 
         if (d < 0) {
-            PyErr_SetString(PyExc_ValueError, "invalid digit found");
-            return NULL;
+            unsigned char c = str[i];
+            return PyErr_Format(PyExc_ValueError, "invalid digit found for "
+                                "base %d, got '%c' (0x%02x)", n, c, c);
         }
         for (j = 0; j < m; j++) {
             k = le ? j : (m - j - 1);
@@ -663,6 +672,32 @@ base2ba(PyObject *module, PyObject *args)
 
 /* ------------------- variable length bitarray format ----------------- */
 
+/* grow buffer by at least one byte */
+static int
+grow_buffer(bitarrayobject *self)
+{
+    size_t newsize = Py_SIZE(self) + 1;
+
+    assert_nbits(self);
+    assert(self->allocated >= Py_SIZE(self));
+    assert(self->ob_exports == 0);
+    assert(self->buffer == NULL);
+    assert(self->readonly == 0);
+
+    /* standard growth pattern */
+    newsize += (newsize >> 4) + (newsize < 8 ? 3 : 7);
+
+    self->ob_item = PyMem_Realloc(self->ob_item, newsize);
+    if (self->ob_item == NULL) {
+        PyErr_NoMemory();
+        return -1;
+    }
+    Py_SET_SIZE(self, newsize);
+    self->allocated = newsize;
+    self->nbits = 8 * newsize;
+    return 0;
+}
+
 /* PADBITS is always 3 - the number of bits that represent the number of
    padding bits.  The actual number of padding bits is called 'padding'
    below, and is in range(0, 7).
@@ -672,37 +707,21 @@ base2ba(PyObject *module, PyObject *args)
    bitarray('000'), which has 5 pad bits. */
 #define PADBITS  3
 
-/* Consume iterator while decoding bytes into bitarray.
-   As we don't have access to bitarrays resize() C function, we give this
-   function a bitarray (large enough in many cases).  We manipulate .nbits
-   and .ob_size (using Py_SET_SIZE) directly without having to call resize().
-   Whenever we need a larger bitarray, we call .frombytes() with a multiple
-   of 7 dummy bytes, such that the added bytes are aligned for the next time
-   we call .frombytes() (to avoid expensive bit shifts).
-   We drop the over-allocated bitarray on the Python side after this function
-   is called.
-*/
+/* consume iterator while decoding bytes into bitarray */
 static PyObject *
 vl_decode(PyObject *module, PyObject *args)
 {
-    const Py_ssize_t ibits = 256;    /* initial number of bits in a */
     PyObject *iter, *item, *a;
     Py_ssize_t padding = 0;  /* number of pad bits read from header byte */
     Py_ssize_t i = 0;        /* bit counter */
     unsigned char b = 0x80;  /* empty stream will raise StopIteration */
     Py_ssize_t k;
 
-    /* Ensure that bits will be aligned when gowing memory below.
-       Possible values for ibits are: 32, 88, 144, 200, 256, 312, ... */
-    assert((ibits + PADBITS) % 7 == 0 && ibits % 8 == 0);
-
     if (!PyArg_ParseTuple(args, "OO", &iter, &a))
         return NULL;
     if (!PyIter_Check(iter))
         return PyErr_Format(PyExc_TypeError, "iterator or bytes expected, "
                             "got '%s'", Py_TYPE(iter)->tp_name);
-    if (ensure_ba_of_length(a, ibits) < 0)
-        return NULL;
 
 #define aa  ((bitarrayobject *) a)
     while ((item = PyIter_Next(iter))) {
@@ -721,19 +740,8 @@ vl_decode(PyObject *module, PyObject *args)
         }
         Py_DECREF(item);
 
-        assert(i == 0 || (i + PADBITS) % 7 == 0);
-        if (i == aa->nbits) {
-            PyObject *ret;  /* return object from .frombytes() call */
-
-            /* grow memory - see above */
-            assert(i % 8 == 0);  /* added dummy bytes are aligned */
-            /* 63 is a multiple of 7 - bytes will be aligned for next call */
-            ret = PyObject_CallMethod(a, "frombytes", BYTES_SIZE_FMT,
-                                      base64_alphabet, (Py_ssize_t) 63);
-            if (ret == NULL)
-                return NULL;
-            Py_DECREF(ret);  /* drop frombytes result */
-        }
+        if (i + 6 >= aa->nbits && grow_buffer(aa) < 0)
+            return NULL;
         assert(i + 6 < aa->nbits);
 
         if (i == 0) {
