@@ -4,44 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "2.2.5"
-
-/* .ob_size is buffer size (in bytes), not the number of elements.
-   The number of elements (bits) is .nbits. */
-typedef struct {
-    PyObject_VAR_HEAD
-    char *ob_item;              /* buffer */
-    Py_ssize_t allocated;       /* allocated buffer size (in bytes) */
-    Py_ssize_t nbits;           /* length of bitarray, i.e. elements */
-    int endian;                 /* bit endianness of bitarray */
-    int ob_exports;             /* how many buffer exports */
-    PyObject *weakreflist;      /* list of weak references */
-} bitarrayobject;
-
-/* --- bit endianness --- */
-#define ENDIAN_LITTLE  0
-#define ENDIAN_BIG     1
-
-/* the endianness string */
-#define ENDIAN_STR(endian)  ((endian) == ENDIAN_LITTLE ? "little" : "big")
-
-/* number of bits that can be stored in given bytes */
-#define BITS(bytes)  ((bytes) << 3)
-
-/* number of bytes necessary to store given bits */
-#define BYTES(bits)  (((bits) + 7) >> 3)
-
-#define BITMASK(endian, i)  \
-    (((char) 1) << ((endian) == ENDIAN_LITTLE ? ((i) & 7) : (7 - ((i) & 7))))
-
-/* assert that .nbits is in agreement with .ob_size */
-#define assert_nbits(self)  assert(BYTES((self)->nbits) == Py_SIZE(self))
-
-/* assert byte index is in range */
-#define assert_byte_in_range(self, j)  \
-    assert(self->ob_item && 0 <= (j) && (j) < Py_SIZE(self))
-
-/* --------------- definitions not specific to bitarray ---------------- */
+#define BITARRAY_VERSION  "2.3.3"
 
 #ifdef STDC_HEADERS
 #include <stddef.h>
@@ -75,6 +38,46 @@ typedef struct {
 #define BYTES_SIZE_FMT  "s#"
 #endif
 
+/* --- bitarrayobject --- */
+
+/* .ob_size is buffer size (in bytes), not the number of elements.
+   The number of elements (bits) is .nbits. */
+typedef struct {
+    PyObject_VAR_HEAD
+    char *ob_item;              /* buffer */
+    Py_ssize_t allocated;       /* allocated buffer size (in bytes) */
+    Py_ssize_t nbits;           /* length of bitarray, i.e. elements */
+    int endian;                 /* bit endianness of bitarray */
+    int ob_exports;             /* how many buffer exports */
+    PyObject *weakreflist;      /* list of weak references */
+    Py_buffer *buffer;          /* used when importing a buffer */
+    int readonly;               /* buffer is readonly */
+} bitarrayobject;
+
+/* --- bit endianness --- */
+#define ENDIAN_LITTLE  0
+#define ENDIAN_BIG     1
+
+/* the endianness string */
+#define ENDIAN_STR(endian)  ((endian) == ENDIAN_LITTLE ? "little" : "big")
+
+/* number of bits that can be stored in given bytes */
+#define BITS(bytes)  ((bytes) << 3)
+
+/* number of bytes necessary to store given bits */
+#define BYTES(bits)  (((bits) + 7) >> 3)
+
+/* we're not using bitmask_table here, as it is actually slower */
+#define BITMASK(self, i)  (((char) 1) << ((self)->endian == ENDIAN_LITTLE ? \
+                                          ((i) % 8) : (7 - (i) % 8)))
+
+/* assert that .nbits is in agreement with .ob_size */
+#define assert_nbits(self)  assert(BYTES((self)->nbits) == Py_SIZE(self))
+
+/* assert byte index is in range */
+#define assert_byte_in_range(self, j)  \
+    assert(self->ob_item && 0 <= (j) && (j) < Py_SIZE(self))
+
 /* ------------ low level access to bits in bitarrayobject ------------- */
 
 static inline int
@@ -83,48 +86,65 @@ getbit(bitarrayobject *self, Py_ssize_t i)
     assert_nbits(self);
     assert_byte_in_range(self, i >> 3);
     assert(0 <= i && i < self->nbits);
-    return (self->ob_item[i >> 3] & BITMASK(self->endian, i) ? 1 : 0);
+    return self->ob_item[i >> 3] & BITMASK(self, i) ? 1 : 0;
 }
 
 static inline void
-setbit(bitarrayobject *self, Py_ssize_t i, int bit)
+setbit(bitarrayobject *self, Py_ssize_t i, int vi)
 {
     char *cp, mask;
 
     assert_nbits(self);
     assert_byte_in_range(self, i >> 3);
     assert(0 <= i && i < self->nbits);
-    mask = BITMASK(self->endian, i);
+    assert(self->readonly == 0);
+
+    mask = BITMASK(self, i);
     cp = self->ob_item + (i >> 3);
-    if (bit)
+    if (vi)
         *cp |= mask;
     else
         *cp &= ~mask;
 }
 
-/* sets unused padding bits (within last byte of buffer) to 0,
-   and return the number of padding bits -- self->nbits is unchanged */
+/* Return the (padded with zeros) last byte of the buffer.  When called with
+   a bitarray whose number of bits is a multiple of 8, return a NUL byte. */
+static inline char
+zeroed_last_byte(bitarrayobject *self)
+{
+    static const char mask_table[2][8] = {
+        {0, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f},  /* little endian */
+        {0, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe},  /* big endian */
+    };
+    Py_ssize_t t = Py_SIZE(self) - 1;  /* index of last byte in buffer */
+    int r = self->nbits % 8;           /* index into mask table */
+
+    if (r == 0)
+        return 0x00;
+    assert_nbits(self);
+    assert_byte_in_range(self, t);
+    return mask_table[self->endian == ENDIAN_BIG][r] & self->ob_item[t];
+}
+
+/* Unless the buffer is readonly, zero out pad bits.
+   Always return the number of pad bits - leave self->nbits unchanged */
 static inline int
 setunused(bitarrayobject *self)
 {
-    const char mask[14] = {
-        0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f,  /* little endian */
-        0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe,  /* big endian */
-    };
-    int i = self->nbits % 8;  /* index into mask array (minus offset) */
+    int r = self->nbits % 8;
 
-    if (i == 0)
+    if (r == 0)
         return 0;
-
-    assert_nbits(self);
-    assert(self->ob_item && Py_SIZE(self) > 0);
-    /* apply the appropriate mask to the last byte in buffer */
-    self->ob_item[Py_SIZE(self) - 1] &=
-        mask[self->endian == ENDIAN_LITTLE ? i - 1 : i + 6];
-
-    return 8 - i;
+    if (self->readonly == 0)
+        self->ob_item[Py_SIZE(self) - 1] = zeroed_last_byte(self);
+    return 8 - r;
 }
 
+static const char bitmask_table[2][8] = {
+    {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80},  /* little endian */
+    {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01},  /* big endian */
+};
+
 static const unsigned char bitcount_lookup[256] = {
 #define B2(n)  n, n + 1, n + 1, n + 2
 #define B4(n)  B2(n), B2(n + 1), B2(n + 1), B2(n + 2)
@@ -135,14 +155,27 @@ static const unsigned char bitcount_lookup[256] = {
 #undef B6
 };
 
+/* normalize index (which may be negative), such that 0 <= i <= n */
+static inline void
+normalize_index(Py_ssize_t n, Py_ssize_t *i)
+{
+    if (*i < 0) {
+        *i += n;
+        if (*i < 0)
+            *i = 0;
+    }
+    if (*i > n)
+        *i = n;
+}
+
 /* Interpret a PyObject (usually PyLong or PyBool) as a bit, return 0 or 1.
    On error, return -1 and set error message. */
 static inline int
-pybit_as_int(PyObject *v)
+pybit_as_int(PyObject *value)
 {
     Py_ssize_t x;
 
-    x = PyNumber_AsSsize_t(v, NULL);
+    x = PyNumber_AsSsize_t(value, NULL);
     if (x == -1 && PyErr_Occurred())
         return -1;
 
